-- 
-- SellPriceTriggerGUI
-- 
-- @Author: Tunis
-- @Date: 30.01.2022
-- @Version: 1.0.1.0
-- 
-- 
-- Changelog:
-- 1.0.1.0
-- Current price and trend (climbing or falling) in table
--

SellPriceTriggerGUI = {}
SellPriceTriggerGUI.CONTROLS = {"sptTitle", "sptHeaderFilltype", "sptHeaderPrice", "sptHeaderGreatDemand", "sptHeaderLowest", "sptHeaderAverage", "sptHeaderHighest" , "sptHeaderTrigger", "sptTriggerItemTemplate", "sptShowHelp", "sptButtonReset", "sptButtonResetGreatDemand", "sptButtonDisable", "sptButtonHighest", "sptButtonAverage", "sptTriggerList", "sptMenuHelpBoxText", "sptMenuHelpBox"}
SellPriceTriggerGUI.color = {
	normal = {1,1,1,1}, --white
	over = {0.9,0.9,0.1,1},
	green = {0.3763,0.6038,0.0782,1},
	red = {0.8069,0.0097,0.0097,1},
	blue = {0.0742,0.4341,0.6939,1},
	transparent = {0,0,0,0}
}
SellPriceTriggerGUI.tableCols = {Image = 1, Filltype = 2, Trend = 3, Current = 4, Great = 5, Lowest = 6, Average = 7, Highest = 8, Trigger = 9}

local SellPriceTriggerGUI_mt = Class(SellPriceTriggerGUI, ScreenElement)

function SellPriceTriggerGUI:new(target)
	local obj = ScreenElement.new(target, SellPriceTriggerGUI_mt)
	obj:registerControls(SellPriceTriggerGUI.CONTROLS)
	return obj
end

function SellPriceTriggerGUI:onCreate()
	self.sptTriggerItemTemplate:unlinkElement()
	self.sptTriggerItemTemplate:setVisible(false)

	self.sptTitle.text = g_i18n:getText('SPT_gui_Title')

	self.sptHeaderFilltype.text = g_i18n:getText('SPT_gui_HeaderFilltype')
	self.sptHeaderPrice.text = g_i18n:getText('SPT_gui_HeaderPrice')
	self.sptHeaderGreatDemand.text = g_i18n:getText('SPT_gui_HeaderGreatDemand')
	self.sptHeaderLowest.text = g_i18n:getText('SPT_gui_HeaderLowest')
	self.sptHeaderAverage.text = g_i18n:getText('SPT_gui_HeaderAverage')
	self.sptHeaderHighest.text = g_i18n:getText('SPT_gui_HeaderHighest')
	self.sptHeaderTrigger.text = g_i18n:getText('SPT_gui_HeaderTrigger')

	self.sptButtonReset:setText(g_i18n:getText('SPT_gui_Reset'))
	self.sptButtonResetGreatDemand:setText(g_i18n:getText('SPT_gui_ResetGreatDemand'))
	self.sptButtonDisable:setText(g_i18n:getText('SPT_gui_Disable'))
	self.sptButtonHighest:setText(g_i18n:getText('SPT_gui_Highest'))
	self.sptButtonAverage:setText(g_i18n:getText('SPT_gui_Average'))
	
	

	self.sptShowHelp.labelElement.text = g_i18n:getText('SPT_gui_ShowHelp')
	self.sptShowHelp.toolTipText = g_i18n:getText('SPT_gui_ShowHelpToolTip')
	
end

function SellPriceTriggerGUI:onOpen()
	SellPriceTriggerGUI:superClass().onOpen(self)
	self:loadList()
	self.updateTimer = 0.0
	self.sptShowHelp:setIsChecked(SellPriceTrigger.showHelp)
end

function SellPriceTriggerGUI:onClose()
	SellPriceTriggerGUI:superClass().onClose(self)
end

function SellPriceTriggerGUI:update(dt)
	SellPriceTriggerGUI:superClass().update(self, dt)
	self.updateTimer = self.updateTimer + dt * g_currentMission.missionInfo.timeScale
	if self.updateTimer >= SellPriceTrigger.updateRun then
		self:changeList()
	end
end

function SellPriceTriggerGUI:onClickBack()
	SellPriceTriggerGUI:superClass().onClickBack(self)
	self:changeScreen(nil)
end

function SellPriceTriggerGUI:onClickOk()
	SellPriceTriggerGUI:superClass().onClickOk(self)

	self:saveList()
	SellPriceTrigger.showHelp = self.sptShowHelp:getIsChecked()
	SellPriceTrigger:updateActionEvents()
	SellPriceTrigger:checkPrices()
	self:onClickBack()
end

function SellPriceTriggerGUI:onSptMenuHelpTextChanged(element)
	if element.text == "" then
		self.sptMenuHelpBox:setVisible(false)
	else
		self.sptMenuHelpBox:setVisible(true)
	end
end

function SellPriceTriggerGUI:onHighlightReset()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_ResetToolTip'))
end

function SellPriceTriggerGUI:onHighlightResetGreatDemand()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_ResetGreatDemandToolTip'))
end

function SellPriceTriggerGUI:onHighlightDisable()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_DisableToolTip'))
end

function SellPriceTriggerGUI:onHighlightHighest()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_HighestToolTip'))
end

function SellPriceTriggerGUI:onHighlightAverage()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_AverageToolTip'))
end

function SellPriceTriggerGUI:onHighlightUp()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_UpToolTip'))
end

function SellPriceTriggerGUI:onHighlightDown()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_DownToolTip'))
end

function SellPriceTriggerGUI:onHighlightRemove()
	self.sptMenuHelpBoxText:setText('')
end

function SellPriceTriggerGUI:onClickReset()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		SellPriceTrigger.triggers[element.trigger.index].highest = 0.0
		SellPriceTrigger.triggers[element.trigger.index].lowest = 0.0
		SellPriceTrigger.triggers[element.trigger.index].mean = 0.0
		SellPriceTrigger.triggers[element.trigger.index].meantotal = 0.0
		SellPriceTrigger.triggers[element.trigger.index].meancount = 0
		SellPriceTrigger:checkPrices()
		SellPriceTrigger:meanPrice(element.trigger.index)
		element.trigger.lowest = SellPriceTrigger.triggers[element.trigger.index].lowest
		element.trigger.mean = SellPriceTrigger.triggers[element.trigger.index].mean
		element.trigger.highest = SellPriceTrigger.triggers[element.trigger.index].highest
		element.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(element.trigger.lowest))
		element.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(element.trigger.mean))
		element.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(element.trigger.highest))
	end
end

function SellPriceTriggerGUI:onClickResetGreatDemand()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		SellPriceTrigger.triggers[element.trigger.index].great = 0.0
		SellPriceTrigger:checkPrices()
		element.trigger.great = SellPriceTrigger.triggers[element.trigger.index].great
		if element.trigger.great > 0 then
			element.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(element.trigger.great))
		else
			element.elements[self.tableCols.Great]:setText('-')
		end
	end
end

function SellPriceTriggerGUI:onClickDisable()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		element.trigger.price = 0
		element.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickHighest()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		element.trigger.price = math.floor(element.trigger.highest)
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickAverage()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		element.trigger.price = math.floor(element.trigger.mean)
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickUp()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		price = price + 1
		local trigger = element.trigger
		element.trigger.price = price
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickDown()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		if price > 0 then
			price = price - 1
			element.trigger.price = price
			if price == 0 then
				element.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
			else
				element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
			end
			self:priceOver(element)
		end
	end
end

function SellPriceTriggerGUI:onClickUpTen()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		price = price + 10
		local trigger = element.trigger
		element.trigger.price = price
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickDownTen()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		if price > 0 then
			price = price - 10
			if price < 0 then price = 0 end
			element.trigger.price = price
			if price == 0 then
				element.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
			else
				element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
			end
			self:priceOver(element)
		end
	end
end

function SellPriceTriggerGUI:loadList()

	self.sptTriggerList:deleteListItems()

	for id,trigger in pairs(SellPriceTrigger.triggers) do
		if trigger.highest > 0 then -- show only filltypes that can be sold
			local new = self.sptTriggerItemTemplate:clone(self.sptTriggerList)
			new:setVisible(true)
			local filltype = g_fillTypeManager.fillTypes[id]
			new.elements[self.tableCols.Image]:setImageFilename(filltype.hudOverlayFilename)
			new.elements[self.tableCols.Filltype]:setText(trigger.title)
			new.elements[self.tableCols.Current]:setText(g_i18n:formatMoney(trigger.current))
			if trigger.great > 0 then
				new.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(trigger.great))
			else
				new.elements[self.tableCols.Great]:setText('-')
			end
			new.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(trigger.lowest))
			new.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(trigger.mean))
			new.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(trigger.highest))
			if trigger.price == 0 then
				new.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
			else
				new.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(trigger.price))
			end
			new:updateAbsolutePosition()
			new.trigger = {}
			new.trigger['index'] = trigger.index
			new.trigger['price'] = trigger.price
			new.trigger['highest'] = trigger.highest
			new.trigger['great'] = trigger.great
			new.trigger['lowest'] = trigger.lowest
			new.trigger['mean'] = trigger.mean

			self:priceOver(new)
			self:priceTrendIcon(new)
		end
	end

end

function SellPriceTriggerGUI:saveList()
	for _,element in pairs(self.sptTriggerList.elements) do
		if element.trigger ~= nil then
			SellPriceTrigger.triggers[element.trigger.index].price = element.trigger.price
			SellPriceTrigger.triggers[element.trigger.index].over = false
			if SellPriceTrigger.debug then
				print("---ListSave "..tostring(element.trigger.index).." price "..tostring(element.trigger.price))
			end
		end
	end
end

function SellPriceTriggerGUI:changeList()
	for _,element in pairs(self.sptTriggerList.elements) do
		if element.trigger ~= nil then
			element.trigger.lowest = SellPriceTrigger.triggers[element.trigger.index].lowest
			element.trigger.mean = SellPriceTrigger.triggers[element.trigger.index].mean
			element.trigger.highest = SellPriceTrigger.triggers[element.trigger.index].highest
			element.elements[self.tableCols.Current]:setText(g_i18n:formatMoney(SellPriceTrigger.triggers[element.trigger.index].current))
			element.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(element.trigger.lowest))
			element.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(element.trigger.mean))
			element.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(element.trigger.highest))
			element.trigger.great = SellPriceTrigger.triggers[element.trigger.index].great
			if element.trigger.great > 0 then
				element.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(element.trigger.great))
			else
				element.elements[self.tableCols.Great]:setText('-')
			end
			self:priceOver(element)
			self:priceTrendIcon(element)
		end
	end
end

function SellPriceTriggerGUI:priceOver(element)
	if(SellPriceTrigger.triggers[element.trigger.index].current > element.trigger.price and element.trigger.price > 0) then
		element.elements[self.tableCols.Trigger].textColor = self.color.over
		element.elements[self.tableCols.Trigger].textSelectedColor = self.color.over
		element.elements[self.tableCols.Trigger].textHighlightedColor = self.color.over
	else
		element.elements[self.tableCols.Trigger].textColor = self.color.normal
		element.elements[self.tableCols.Trigger].textSelectedColor = self.color.normal
		element.elements[self.tableCols.Trigger].textHighlightedColor = self.color.normal
	end
end

function SellPriceTriggerGUI:priceTrendIcon(element)
	local PRICE_CLIMBING = 4
	local PRICE_FALLING = 2
	local PRICE_GREAT_DEMAND = 5
	
	if SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_FALLING then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.red[1],self.color.red[2],self.color.red[3],self.color.red[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(3.1416)
	elseif SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_CLIMBING then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.green[1],self.color.green[2],self.color.green[3],self.color.green[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(0)
	elseif SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_GREAT_DEMAND then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.blue[1],self.color.blue[2],self.color.blue[3],self.color.blue[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(1.5708)
	else
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.transparent[1],self.color.transparent[2],self.color.transparent[3],self.color.transparent[4])
	end
end
