-- 
-- SellPriceTrigger
-- 
-- @Author: Tunis
-- @Date: 30.01.2022
-- @Version: 1.0.1.0
-- 
-- 
-- Changelog:
-- 1.0.1.0
-- Current price and trend (climbing or falling)
--

SellPriceTrigger = {}
SellPriceTrigger.moddir = g_currentModDirectory
SellPriceTrigger.triggers = {} -- All settings/triggers
SellPriceTrigger.NOTIFICATION_TRIGGER = {0.1,0.1,0.5,1} -- Notification color

SellPriceTrigger.debug = false

addModEventListener(SellPriceTrigger)

function SellPriceTrigger:loadMap()

	BaseMission.unregisterActionEvents = Utils.appendedFunction(BaseMission.unregisterActionEvents, SellPriceTrigger.unregisterActionEvents)
	FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, SellPriceTrigger.registerActionEvents)

	-- SaveSettings
	FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, SellPriceTrigger.saveCfg)

	--Init
	self.updateTimer = 0.0
	self.updateRun = 3000
	self.updateEvery = 60000
	self.eventName = {}
	self.showHelp = true
	self.soundLevel = 0.8
	self:fillTypes()
	self:loadCfg()
	
	--Sound
	self.sound = createSample("TriggerSound")
	local file = "data/sounds/ui/uiTransaction.wav"
	loadSample(self.sound, file, false)

	--gui
	g_gui:loadProfiles(self.moddir .. "gui/guiProfiles.xml")
	g_gui:loadGui(self.moddir .. "gui/SellPriceTriggerGUI.xml", "SellPriceTriggerGUI", SellPriceTriggerGUI:new())

	-- mean calc
	g_messageCenter:subscribe(MessageType.HOUR_CHANGED, self.hourChanged, self)

end

function SellPriceTrigger:checkPrices(mean)

	mean = mean or false
	--mean = true -- test

	local prices = {}

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do

		if station.isSellingPoint ~= nil and station.isSellingPoint == true then
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true then

					local price = station:getEffectiveFillTypePrice(fillType)*1000
					local greatDemand = false
					if station.greatDemandFillType ~= nil and station.greatDemandFillType == fillType then
						greatDemand = true
					end

					if prices[fillType] ~= nil then
						if price > prices[fillType].price then
							prices[fillType].price = price
							prices[fillType].stationName = station.uiName
							prices[fillType].greatDemand = greatDemand
							prices[fillType].priceTrend = station:getCurrentPricingTrend(fillType)
						end
						if price < prices[fillType].minprice then
							prices[fillType].minprice = price
						end
					else
						prices[fillType] = {price=0.0,priceTrend=0,stationName="",greatDemand=false,minprice=0.0,meantotal=0.0,meancount=0}
						prices[fillType].price = price
						prices[fillType].minprice = price
						prices[fillType].stationName = station.uiName
						prices[fillType].greatDemand = greatDemand
						prices[fillType].priceTrend = station:getCurrentPricingTrend(fillType)
					end

					if mean == true and price > 0 and greatDemand == false then -- exclude great demand from mean
							prices[fillType].meantotal = prices[fillType].meantotal + price
							prices[fillType].meancount = prices[fillType].meancount + 1
					end

				end
			end
		end
	end

	for fillType,trigger in pairs(self.triggers) do

		if prices[fillType] ~= nil then
			if trigger.price > 0 and prices[fillType].price > trigger.price then
				if trigger.over == false or trigger.station ~= prices[fillType].stationName then
					trigger.station = prices[fillType].stationName
					local mess = ('%s %s @ %s'):format(trigger.title, g_i18n:formatMoney(prices[fillType].price), prices[fillType].stationName)
					g_currentMission:addIngameNotification(self.NOTIFICATION_TRIGGER, mess, "SPTupdate")
					playSample(self.sound, 1, self.soundLevel, 1, 0, 0)
					trigger.over = true
				end
			else
				if trigger.over == true then
					trigger.over = false
				end
			end

			if prices[fillType].greatDemand == true then
				if prices[fillType].price > trigger.great then
					trigger.great = prices[fillType].price
				end	
			else
				if prices[fillType].price > trigger.highest then
					trigger.highest = prices[fillType].price
				end
			end

			if prices[fillType].minprice < trigger.lowest or trigger.lowest == 0 then
				trigger.lowest = prices[fillType].minprice
			end

			if mean == true and prices[fillType].meantotal > 0 then
				trigger.meantotal = trigger.meantotal + (prices[fillType].meantotal / prices[fillType].meancount)
				trigger.meancount = trigger.meancount + 1
				trigger.mean = trigger.meantotal / trigger.meancount
			end

			trigger.current = prices[fillType].price
			trigger.currentTrend = prices[fillType].priceTrend

		end

	end

	self.updateTimer = 0

end

function SellPriceTrigger:meanPrice(id) -- id = fillType

	local prices = {}

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do

		if station.isSellingPoint ~= nil and station.isSellingPoint == true then
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true and fillType == id then

					local price = station:getEffectiveFillTypePrice(fillType)*1000
					local greatDemand = false
					if station.greatDemandFillType ~= nil and station.greatDemandFillType == fillType then
						greatDemand = true
					end

					if prices[fillType] == nil then
						prices[fillType] = {meantotal=0.0,meancount=0}
					end

					if price > 0 and greatDemand == false then
							prices[fillType].meantotal = prices[fillType].meantotal + price
							prices[fillType].meancount = prices[fillType].meancount + 1
					end

				end
			end
		end
	end

	if self.triggers[id] ~= nil and prices[id] ~= nil then
		if prices[id].meantotal > 0 then
			self.triggers[id].meantotal = self.triggers[id].meantotal + (prices[id].meantotal / prices[id].meancount)
			self.triggers[id].meancount = self.triggers[id].meancount + 1
			self.triggers[id].mean = self.triggers[id].meantotal / self.triggers[id].meancount
		end
	end

end

function SellPriceTrigger:fillTypes()
	for _,filltypes in pairs(g_fillTypeManager.fillTypes) do
		if filltypes.showOnPriceTable ~= nil and filltypes.showOnPriceTable == true then
			if self.debug then
				print("---- "..tostring(filltypes.index).." "..tostring(filltypes.name).." "..tostring(filltypes.title))
			end
			self.triggers[filltypes.index] = {index="",name="",title="",price=0,lowest=0.0,mean=0.0,highest=0.0,great=0.0,station="",over=false,meantotal=0.0,meancount=0,current=0.0,currentTrend=0}
			self.triggers[filltypes.index].index = filltypes.index
			self.triggers[filltypes.index].name = filltypes.name
			self.triggers[filltypes.index].title = filltypes.title
		end
	end
end

function SellPriceTrigger:loadCfg()
	if g_currentMission.missionInfo.savegameDirectory ~= nil then
		local file = g_currentMission.missionInfo.savegameDirectory .. "/sellpricetrigger.xml"
		if fileExists(file) then
			local XML = loadXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
			if self.debug then
				print("---Load XML")
			end

			if getXMLBool(XML, "sellPriceTriggers#showhelp") ~= nil then self.showHelp = getXMLBool(XML, "sellPriceTriggers#showhelp") end
			
			if getXMLFloat(XML, "sellPriceTriggers#R") ~= nil and getXMLFloat(XML, "sellPriceTriggers#G") ~= nil and getXMLFloat(XML, "sellPriceTriggers#B") ~= nil then
				local R = getXMLFloat(XML, "sellPriceTriggers#R")
				local G = getXMLFloat(XML, "sellPriceTriggers#G")
				local B = getXMLFloat(XML, "sellPriceTriggers#B")
				self.NOTIFICATION_TRIGGER = {R,G,B,1}
				if self.debug then
					print("---Load R "..tostring(R).." G "..tostring(G).." B "..tostring(B))
				end
			end
			
			if getXMLFloat(XML, "sellPriceTriggers#soundlevel") ~= nil then self.soundLevel = getXMLFloat(XML, "sellPriceTriggers#soundlevel") end

			for x=0,1000 do

				local groupNameTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
				local name = getXMLString(XML, groupNameTag.. "#name")
				if self.debug then
					print("---XML "..tostring(x).." name "..tostring(name))
				end

				if name == nil then
					break
				end

				if self.triggers[g_fillTypeManager.nameToIndex[name]] ~= nil then
					if self.debug then
						print("---XML found name "..tostring(name))
					end
					if getXMLInt(XML, groupNameTag.. "#price") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].price = getXMLInt(XML, groupNameTag.. "#price") end
					if getXMLFloat(XML, groupNameTag.. "#highest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].highest = getXMLFloat(XML, groupNameTag.. "#highest") end
					if getXMLFloat(XML, groupNameTag.. "#great") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].great = getXMLFloat(XML, groupNameTag.. "#great") end
					if getXMLFloat(XML, groupNameTag.. "#lowest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].lowest = getXMLFloat(XML, groupNameTag.. "#lowest") end
					if getXMLFloat(XML, groupNameTag.. "#mean") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].mean = getXMLFloat(XML, groupNameTag.. "#mean") end
					if getXMLFloat(XML, groupNameTag.. "#meantotal") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].meantotal = getXMLFloat(XML, groupNameTag.. "#meantotal") end
					if getXMLInt(XML, groupNameTag.. "#meancount") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].meancount = getXMLInt(XML, groupNameTag.. "#meancount") end
				end

			end
		end
	end
end

function SellPriceTrigger:saveCfg()
	if g_currentMission.missionInfo.savegameDirectory ~= nil then
		local file = g_currentMission.missionInfo.savegameDirectory.. "/sellpricetrigger.xml"
		local XML = createXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
		local x = 0
		if SellPriceTrigger.debug then
			print("---Save XML")
		end
		
		setXMLBool(XML, "sellPriceTriggers#showhelp", SellPriceTrigger.showHelp)
		setXMLFloat(XML, "sellPriceTriggers#R", SellPriceTrigger.NOTIFICATION_TRIGGER[1])
		setXMLFloat(XML, "sellPriceTriggers#G", SellPriceTrigger.NOTIFICATION_TRIGGER[2])
		setXMLFloat(XML, "sellPriceTriggers#B", SellPriceTrigger.NOTIFICATION_TRIGGER[3])
		setXMLFloat(XML, "sellPriceTriggers#soundlevel", SellPriceTrigger.soundLevel)
		for fillType,trigger in pairs(SellPriceTrigger.triggers) do

			local groupNameTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
			setXMLString(XML, groupNameTag.. "#name", trigger.name)
			setXMLInt(XML, groupNameTag.. "#price", trigger.price)
			setXMLFloat(XML, groupNameTag.. "#highest", trigger.highest)
			setXMLFloat(XML, groupNameTag.. "#great", trigger.great)
			setXMLFloat(XML, groupNameTag.. "#lowest", trigger.lowest)
			setXMLFloat(XML, groupNameTag.. "#mean", trigger.mean)
			setXMLFloat(XML, groupNameTag.. "#meantotal", trigger.meantotal)
			setXMLInt(XML, groupNameTag.. "#meancount", trigger.meancount)
			x=x+1

		end

		saveXMLFile(XML)
	end
end

function SellPriceTrigger:loadSavegame() end
function SellPriceTrigger:saveSavegame() end

function SellPriceTrigger:update(dt)
	self.updateTimer = self.updateTimer + dt * g_currentMission.missionInfo.timeScale
	if self.updateTimer >= self.updateRun then
		self.updateRun = self.updateEvery
		self:checkPrices()
	end
end

function SellPriceTrigger:hourChanged()
		if SellPriceTrigger.debug then
			print("---hourChanged "..tostring(g_currentMission.environment.currentHour))
		end
		SellPriceTrigger:checkPrices(true)
end

function SellPriceTrigger:mouseEvent(posX, posY, isDown, isUp, button) end
function SellPriceTrigger:keyEvent(unicode, sym, modifier, isDown) end
function SellPriceTrigger:draw() end
function SellPriceTrigger:delete() end
function SellPriceTrigger:deleteMap() end

function SellPriceTrigger:registerActionEvents()
	local _, eventId = g_gui.inputManager:registerActionEvent(InputAction.SPT_SHOW_UI, self, SellPriceTrigger.actionEventShowUI, false, true, false, true)
	SellPriceTrigger.eventId = eventId
	self.inputManager:setActionEventTextVisibility(eventId, SellPriceTrigger.showHelp)
end

function SellPriceTrigger:unregisterActionEvents()
	g_gui.inputManager:removeActionEventsByTarget(self)
end

function SellPriceTrigger:updateActionEvents()
	--Update Show Help
	g_gui.inputManager:setActionEventTextVisibility(SellPriceTrigger.eventId, SellPriceTrigger.showHelp)
end

function SellPriceTrigger:actionEventShowUI()
	if g_gui:getIsGuiVisible() then
		return 
	end
	g_gui:showGui("SellPriceTriggerGUI")
end
